@@ -2,10 +2,11 @@
 
 use std::{
     error::Error,
-    fmt::{Display, Formatter, Result as FmtResult},
+    fmt::{Debug, Display, Formatter, Result as FmtResult},
 };
 use twilight_model::application::component::{
-    select_menu::SelectMenuOption, Component, ComponentType,
+    button::ButtonStyle, select_menu::SelectMenuOption, ActionRow, Button, Component,
+    ComponentType, SelectMenu, TextInput,
 };
 
 /// Maximum number of [`Component`]s allowed inside an [`ActionRow`].
@@ -124,7 +125,7 @@ pub const SELECT_OPTION_VALUE_LENGTH: usize = 100;
 ///
 /// [`SelectMenu::placeholder`]: twilight_model::application::component::select_menu::SelectMenu::placeholder
 /// [1]: https://discord.com/developers/docs/interactions/message-components#select-menu-object-select-menu-structure
-pub const SELECT_PLACEHOLDER_LENGTH: usize = 100;
+pub const SELECT_PLACEHOLDER_LENGTH: usize = 150;
 
 /// Maximum length of [`TextInput::value`].
 ///
@@ -197,6 +198,22 @@ impl Display for ComponentValidationError {
 
                 Display::fmt(&ACTION_ROW_COMPONENT_COUNT, f)
             }
+            ComponentValidationErrorType::ButtonConflict => {
+                f.write_str("button has both a custom id and url, which is never valid")
+            }
+            ComponentValidationErrorType::ButtonStyle { style } => {
+                f.write_str("button has a type of ")?;
+                Debug::fmt(style, f)?;
+                f.write_str(", which must have a ")?;
+
+                f.write_str(if *style == ButtonStyle::Link {
+                    "url"
+                } else {
+                    "custom id"
+                })?;
+
+                f.write_str(" configured")
+            }
             ComponentValidationErrorType::ComponentCount { count } => {
                 Display::fmt(count, f)?;
                 f.write_str(" components were provided, but the max is ")?;
@@ -328,6 +345,16 @@ pub enum ComponentValidationErrorType {
         /// Number of components within the action row.
         count: usize,
     },
+    /// Button has both a custom ID and URL set.
+    ButtonConflict,
+    /// Button does not have the required field based on its style.
+    ///
+    /// A button with a style of [`ButtonStyle::Link`] must have a URL set,
+    /// while buttons of other styles must have a custom ID set.
+    ButtonStyle {
+        /// Style of the button.
+        style: ButtonStyle,
+    },
     /// Number of components provided is larger than
     /// [the maximum][`COMPONENT_COUNT`].
     ComponentCount {
@@ -430,19 +457,146 @@ pub enum ComponentValidationErrorType {
     },
 }
 
-/// Ensure a component is correct.
+/// Ensure that a top-level request component is correct.
+///
+/// Intended to ensure that a fully formed top-level component for requests
+/// is an action row.
+///
+/// Refer to other validators like [`button`] if you need to validate other
+/// components.
 ///
 /// # Errors
 ///
-/// Returns an error of type [`ActionRowComponentCount`] if the provided list of
-/// components is too many for an [`ActionRow`].
+/// Returns an error of type [`InvalidRootComponent`] if the component is not an
+/// [`ActionRow`].
+///
+/// Refer to [`action_row`] for potential errors when validating an action row
+/// component.
+///
+/// [`InvalidRootComponent`]: ComponentValidationErrorType::InvalidRootComponent
+pub fn component(component: &Component) -> Result<(), ComponentValidationError> {
+    match component {
+        Component::ActionRow(action_row) => self::action_row(action_row)?,
+        other => {
+            return Err(ComponentValidationError {
+                kind: ComponentValidationErrorType::InvalidRootComponent { kind: other.kind() },
+            });
+        }
+    }
+
+    Ok(())
+}
+
+/// Ensure that an action row is correct.
+///
+/// # Errors
+///
+/// Returns an error of type [`ActionRowComponentCount`] if the action row has
+/// too many components in it.
 ///
 /// Returns an error of type [`InvalidChildComponent`] if the provided nested
 /// component is an [`ActionRow`]. Action rows can not contain another action
 /// row.
 ///
-/// Returns an error of type [`InvalidRootComponent`] if the root component is
-/// not an [`ActionRow`].
+/// Refer to [`button`] for potential errors when validating a button in the
+/// action row.
+///
+/// Refer to [`select_menu`] for potential errors when validating a select menu
+/// in the action row.
+///
+/// Refer to [`text_input`] for potential errors when validating a text input in
+/// the action row.
+///
+/// [`ActionRowComponentCount`]: ComponentValidationErrorType::ActionRowComponentCount
+/// [`InvalidChildComponent`]: ComponentValidationErrorType::InvalidChildComponent
+pub fn action_row(action_row: &ActionRow) -> Result<(), ComponentValidationError> {
+    self::component_action_row_components(&action_row.components)?;
+
+    for component in &action_row.components {
+        match component {
+            Component::ActionRow(_) => {
+                return Err(ComponentValidationError {
+                    kind: ComponentValidationErrorType::InvalidChildComponent {
+                        kind: ComponentType::ActionRow,
+                    },
+                });
+            }
+            Component::Button(button) => self::button(button)?,
+            Component::SelectMenu(select_menu) => self::select_menu(select_menu)?,
+            Component::TextInput(text_input) => self::text_input(text_input)?,
+        }
+    }
+
+    Ok(())
+}
+
+/// Ensure that a button is correct.
+///
+/// # Errors
+///
+/// Returns an error of type [`ButtonConflict`] if both a custom ID and URL are
+/// specified.
+///
+/// Returns an error of type
+/// [`ButtonStyle`][`ComponentValidationErrorType::ButtonStyle`] if
+/// [`ButtonStyle::Link`] is provided and a URL is provided, or if the style is
+/// not [`ButtonStyle::Link`] and a custom ID is not provided.
+///
+/// Returns an error of type [`ComponentCustomIdLength`] if the provided custom
+/// ID is too long.
+///
+/// Returns an error of type [`ComponentLabelLength`] if the provided button
+/// label is too long.
+///
+/// [`ButtonConflict`]: ComponentValidationErrorType::ButtonConflict
+/// [`ComponentCustomIdLength`]: ComponentValidationErrorType::ComponentCustomIdLength
+/// [`ComponentLabelLength`]: ComponentValidationErrorType::ComponentLabelLength
+pub fn button(button: &Button) -> Result<(), ComponentValidationError> {
+    let has_custom_id = button.custom_id.is_some();
+    let has_url = button.url.is_some();
+
+    // First check if a custom ID and URL are both set. If so this
+    // results in a conflict, as no valid button may have both set.
+    if has_custom_id && has_url {
+        return Err(ComponentValidationError {
+            kind: ComponentValidationErrorType::ButtonConflict,
+        });
+    }
+
+    // Next, we check if the button is a link and a URL is not set.
+    //
+    // Lastly, we check if the button is not a link and a custom ID is
+    // not set.
+    let is_link = button.style == ButtonStyle::Link;
+
+    if (is_link && !has_url) || (!is_link && !has_custom_id) {
+        return Err(ComponentValidationError {
+            kind: ComponentValidationErrorType::ButtonStyle {
+                style: button.style,
+            },
+        });
+    }
+
+    if let Some(custom_id) = button.custom_id.as_ref() {
+        self::component_custom_id(custom_id)?;
+    }
+
+    if let Some(label) = button.label.as_ref() {
+        self::component_label(label)?;
+    }
+
+    Ok(())
+}
+
+/// Ensure that a select menu is correct.
+///
+/// # Errors
+///
+/// Returns an error of type [`ComponentCustomIdLength`] if the provided custom
+/// ID is too long.
+///
+/// Returns an error of type [`ComponentLabelLength`] if the provided button
+/// label is too long.
 ///
 /// Returns an error of type [`SelectMaximumValuesCount`] if the provided number
 /// of select menu values that can be chosen is smaller than the minimum or
@@ -463,106 +617,86 @@ pub enum ComponentValidationErrorType {
 /// Returns an error of type [`SelectPlaceholderLength`] if a provided select
 /// placeholder is too long.
 ///
-/// [`ActionRowComponentCount`]: ComponentValidationErrorType::ActionRowComponentCount
-/// [`ActionRow`]: twilight_model::application::component::ActionRow
-/// [`InvalidChildComponent`]: ComponentValidationErrorType::InvalidChildComponent
-/// [`InvalidRootComponent`]: ComponentValidationErrorType::InvalidRootComponent
+/// [`ComponentCustomIdLength`]: ComponentValidationErrorType::ComponentCustomIdLength
+/// [`ComponentLabelLength`]: ComponentValidationErrorType::ComponentLabelLength
 /// [`SelectMaximumValuesCount`]: ComponentValidationErrorType::SelectMaximumValuesCount
 /// [`SelectMinimumValuesCount`]: ComponentValidationErrorType::SelectMinimumValuesCount
 /// [`SelectOptionDescriptionLength`]: ComponentValidationErrorType::SelectOptionDescriptionLength
 /// [`SelectOptionLabelLength`]: ComponentValidationErrorType::SelectOptionLabelLength
 /// [`SelectOptionValueLength`]: ComponentValidationErrorType::SelectOptionValueLength
 /// [`SelectPlaceholderLength`]: ComponentValidationErrorType::SelectPlaceholderLength
-pub fn component(component: &Component) -> Result<(), ComponentValidationError> {
-    match component {
-        Component::ActionRow(action_row) => {
-            self::component_action_row_components(&action_row.components)?;
+pub fn select_menu(select_menu: &SelectMenu) -> Result<(), ComponentValidationError> {
+    self::component_custom_id(&select_menu.custom_id)?;
+    self::component_select_options(&select_menu.options)?;
 
-            for inner in &action_row.components {
-                self::component_inner(inner)?;
-            }
-        }
-        other => {
-            return Err(ComponentValidationError {
-                kind: ComponentValidationErrorType::InvalidRootComponent { kind: other.kind() },
-            });
+    if let Some(placeholder) = select_menu.placeholder.as_ref() {
+        self::component_select_placeholder(placeholder)?;
+    }
+
+    if let Some(max_values) = select_menu.max_values {
+        self::component_select_max_values(usize::from(max_values))?;
+    }
+
+    if let Some(min_values) = select_menu.min_values {
+        self::component_select_min_values(usize::from(min_values))?;
+    }
+
+    for option in &select_menu.options {
+        self::component_select_option_label(&option.label)?;
+        self::component_select_option_value(&option.value)?;
+
+        if let Some(description) = option.description.as_ref() {
+            self::component_option_description(description)?;
         }
     }
 
     Ok(())
 }
 
-/// Validate the contents of a component that is within another component, i.e.
-/// one that is not a root component.
+/// Ensure that a text input is correct.
 ///
 /// # Errors
 ///
-/// Refer to the errors section of [`component`] for a list of errors that may
-/// occur.
-fn component_inner(component: &Component) -> Result<(), ComponentValidationError> {
-    match component {
-        Component::ActionRow(_) => {
-            return Err(ComponentValidationError {
-                kind: ComponentValidationErrorType::InvalidChildComponent {
-                    kind: ComponentType::ActionRow,
-                },
-            })
-        }
-        Component::Button(button) => {
-            if let Some(custom_id) = button.custom_id.as_ref() {
-                self::component_custom_id(custom_id)?;
-            }
+/// Returns an error of type [`ComponentCustomIdLength`] if the provided custom
+/// ID is too long.
+///
+/// Returns an error of type [`ComponentLabelLength`] if the provided button
+/// label is too long.
+///
+/// Returns an error of type [`TextInputMaxLength`] if the length is invalid.
+///
+/// Returns an error of type [`TextInputMinLength`] if the length is invalid.
+///
+/// Returns an error of type [`TextInputPlaceholderLength`] if the provided
+/// placeholder is too long.
+///
+/// Returns an error of type [`TextInputValueLength`] if the length is invalid.
+///
+/// [`ComponentCustomIdLength`]: ComponentValidationErrorType::ComponentCustomIdLength
+/// [`ComponentLabelLength`]: ComponentValidationErrorType::ComponentLabelLength
+/// [`TextInputMaxLength`]: ComponentValidationErrorType::TextInputMaxLength
+/// [`TextInputMinLength`]: ComponentValidationErrorType::TextInputMinLength
+/// [`TextInputPlaceholderLength`]: ComponentValidationErrorType::TextInputPlaceholderLength
+/// [`TextInputValueLength`]: ComponentValidationErrorType::TextInputValueLength
+pub fn text_input(text_input: &TextInput) -> Result<(), ComponentValidationError> {
+    self::component_custom_id(&text_input.custom_id)?;
 
-            if let Some(label) = button.label.as_ref() {
-                self::component_label(label)?;
-            }
-        }
-        Component::SelectMenu(select_menu) => {
-            self::component_custom_id(&select_menu.custom_id)?;
-            self::component_select_options(&select_menu.options)?;
+    self::component_label(&text_input.label)?;
 
-            if let Some(placeholder) = select_menu.placeholder.as_ref() {
-                self::component_select_placeholder(placeholder)?;
-            }
+    if let Some(max_length) = text_input.max_length {
+        self::component_text_input_max(max_length)?;
+    }
 
-            if let Some(max_values) = select_menu.max_values {
-                self::component_select_max_values(usize::from(max_values))?;
-            }
+    if let Some(min_length) = text_input.min_length {
+        self::component_text_input_min(min_length)?;
+    }
 
-            if let Some(min_values) = select_menu.min_values {
-                self::component_select_min_values(usize::from(min_values))?;
-            }
+    if let Some(placeholder) = text_input.placeholder.as_ref() {
+        self::component_text_input_placeholder(placeholder)?;
+    }
 
-            for option in &select_menu.options {
-                self::component_select_option_label(&option.label)?;
-                self::component_select_option_value(&option.value)?;
-
-                if let Some(description) = option.description.as_ref() {
-                    self::component_option_description(description)?;
-                }
-            }
-        }
-        Component::TextInput(text_input) => {
-            self::component_custom_id(&text_input.custom_id)?;
-
-            self::component_label(&text_input.label)?;
-
-            if let Some(max_length) = text_input.max_length {
-                self::component_text_input_max(max_length)?;
-            }
-
-            if let Some(min_length) = text_input.min_length {
-                self::component_text_input_min(min_length)?;
-            }
-
-            if let Some(placeholder) = text_input.placeholder.as_ref() {
-                self::component_text_input_placeholder(placeholder)?;
-            }
-
-            if let Some(value) = text_input.value.as_ref() {
-                self::component_text_input_value(value)?;
-            }
-        }
+    if let Some(value) = text_input.value.as_ref() {
+        self::component_text_input_value(value)?;
     }
 
     Ok(())
@@ -911,16 +1045,40 @@ mod tests {
     assert_impl_all!(ComponentValidationErrorType: Debug, Send, Sync);
     assert_impl_all!(ComponentValidationError: Debug, Send, Sync);
 
+    // All styles of buttons.
+    const fn all_button_styles() -> &'static [ButtonStyle] {
+        const BUTTON_STYLES: &[ButtonStyle] = &[
+            ButtonStyle::Primary,
+            ButtonStyle::Secondary,
+            ButtonStyle::Success,
+            ButtonStyle::Danger,
+            ButtonStyle::Link,
+        ];
+
+        // No-op match to ensure we've registered all styles.
+        //
+        // If a new variant has been added please add it to the above constant.
+        match ButtonStyle::Primary {
+            ButtonStyle::Primary
+            | ButtonStyle::Secondary
+            | ButtonStyle::Success
+            | ButtonStyle::Danger
+            | ButtonStyle::Link => {}
+        }
+
+        BUTTON_STYLES
+    }
+
     #[test]
     fn test_component() {
         let button = Button {
-            custom_id: Some("custom id 1".into()),
+            custom_id: None,
             disabled: false,
             emoji: Some(ReactionType::Unicode {
                 name: "📚".into()
             }),
             label: Some("Read".into()),
-            style: ButtonStyle::Danger,
+            style: ButtonStyle::Link,
             url: Some("https://abebooks.com".into()),
         };
 
@@ -939,18 +1097,18 @@ mod tests {
             placeholder: Some("Choose a book".into()),
         };
 
-        let action_row = Component::ActionRow(ActionRow {
+        let action_row = ActionRow {
             components: Vec::from([
                 Component::SelectMenu(select_menu.clone()),
                 Component::Button(button),
             ]),
-        });
+        };
 
-        assert!(component(&action_row).is_ok());
+        assert!(component(&Component::ActionRow(action_row.clone())).is_ok());
 
         assert!(component(&Component::SelectMenu(select_menu.clone())).is_err());
 
-        assert!(component_inner(&action_row).is_err());
+        assert!(super::action_row(&action_row).is_ok());
 
         let invalid_action_row = Component::ActionRow(ActionRow {
             components: Vec::from([
@@ -966,6 +1124,53 @@ mod tests {
         assert!(component(&invalid_action_row).is_err());
     }
 
+    // Test that a button with both a custom ID and URL results in a
+    // [`ComponentValidationErrorType::ButtonConflict`] error type.
+    #[test]
+    fn test_button_conflict() {
+        let button = Button {
+            custom_id: Some("a".to_owned()),
+            disabled: false,
+            emoji: None,
+            label: None,
+            style: ButtonStyle::Primary,
+            url: Some("https://twilight.rs".to_owned()),
+        };
+
+        assert!(matches!(
+            super::button(&button),
+            Err(ComponentValidationError {
+                kind: ComponentValidationErrorType::ButtonConflict,
+            }),
+        ));
+    }
+
+    // Test that all button styles with no custom ID or URL results in a
+    // [`ComponentValidationErrorType::ButtonStyle`] error type.
+    #[test]
+    fn test_button_style() {
+        for style in all_button_styles().iter() {
+            let button = Button {
+                custom_id: None,
+                disabled: false,
+                emoji: None,
+                label: Some("some label".to_owned()),
+                style: *style,
+                url: None,
+            };
+
+            assert!(matches!(
+                super::button(&button),
+                Err(ComponentValidationError {
+                    kind: ComponentValidationErrorType::ButtonStyle {
+                        style: error_style,
+                    }
+                })
+                if error_style == *style
+            ));
+        }
+    }
+
     #[test]
     fn test_component_label() {
         assert!(component_label("").is_ok());
@@ -1053,9 +1258,9 @@ mod tests {
     fn test_component_select_placeholder() {
         assert!(component_select_placeholder("").is_ok());
         assert!(component_select_placeholder("a").is_ok());
-        assert!(component_select_placeholder("a".repeat(100)).is_ok());
+        assert!(component_select_placeholder("a".repeat(150)).is_ok());
 
-        assert!(component_select_placeholder("a".repeat(101)).is_err());
+        assert!(component_select_placeholder("a".repeat(151)).is_err());
     }
 
     #[test]
